// c8051 rx 433hhz for RC torpedo
// JLC 10/02/2021

//-----------------------------------------------------------------------------
// MACROS
//-----------------------------------------------------------------------------

#define SYSCLK       24500000     // SYSCLK frequency in Hz 
#define BAUDRATE     9600         // Baud rate of UART in bps 
#define FRAMESIZE 6               //  StartByte Rudder Motor Activation CRC EndByte
#define DATASIZE (FRAMESIZE - 2)  //  Rudder Motor Activation CRC

#define PULSMID  0xAA    		// 85   170=0xAA   255
#define MAX  	 1360			//  1360*0.5 = 670 uS
#define MID  	 680			//  680*0.5  = 340 uS
#define DZ       15				// servo dead zone (30)
#define DCstall 0xF0
#define NEUTRAL 0x00
#define RIGHT   0x01
#define LEFT    0x02
#define MOT		0x00
#define SERV   	0x01
#define ACTIV   0x02
#define CRC		0x03
#define SYNCHRO 0x00	// synchronisation = chanel 0
#define STDCH   0x01    // standard chanel
#define SWCH	0x02    // multiswitch chanel


/*------ transmiter dependant part
 ppm chanel position on multiswitch ref. 8101
 _____________
|  5  6	 7  8 |
|  1  2  3  4 |
|_____________|

*/

#define MAXCHANEL 	0x08   	// FC18 8  FC28 9
#define MSWITCH 	0x08  	// multiswitch chanel# 
#define MSW_ON 	  	0x01 	// multiswitch  0 off 1 on
#define MOTOR		0x07   	// chanel# for motor (0 = none)
#define SERVO   	0x01  	// chanel# for magnet servo (0 = none)
//#define MOTOR_SW 	0x02 	// multiswitch chanel# for motor (0 = none)
//#define SERVO_SW 	0x01   	// multiswitch chanel# for magnet servo (0 = none)
//#define ACTIF1   	0x05 	// chanel# for torpedo1 activation
//#define ACTIF2   	0x06   	// chanel# for torpedo2 activation
#define ACTIF1_SW 0x04  	// multiswitch chanel# for torpedo1 activation
#define ACTIF2_SW 0x03  	// multiswitch chanel# for torpedo2 activation
//---------- end dependant part


#define TORP1 0x01 // 00000001
#define TORP2 0x02 // 00000010
#define TORP3 0x04 // 00000100
#define TORP4 0x08 // 00001000

// x8 + x4 + x3
#define CRC_POLY 0x8C  

const unsigned char StartByte = 3;
const unsigned char EndByte   = 5;

//-----------------------------------------------------------------------------
// GLOBAL VARIABLES IN RAM
//-----------------------------------------------------------------------------

sfr16 T2 = 0xCC;		// timer2
sfr16 T2RL = 0xCA;		// timer2 reload
//sbit P04 = P0^4;
sbit G1  = 0x82;		// P02 Gate Servo PWM 		digital output (push pull)
sbit G2  = 0x83;		// P03 Gate Servo PWM 		digital output (push pull)

// ISR vars
volatile bit tx_busy = 0; // Uart transmission in progress for full duplex
volatile bit NewFrame = 0;
volatile unsigned char BYTEinCount = 0;
volatile unsigned char uart_rx_buf[DATASIZE]; // StartByte Rudder Motor Activation CRC EndByte

unsigned char tmp_arr[DATASIZE];
unsigned char ID;			// torpedo identifier
unsigned char active;		// torpedos activation byte on/off
unsigned char sdir;			// servo direction: 0 neutral  1 right 2 left
unsigned char DCM;			// motor duty cycle:  0x55 100%     0xFF 0%
unsigned char DCS;			// rudder (magnet servo) duty cycle:  0x55 100%     0xFF 0%
unsigned char tmp;       	// for shifted pulse

//-----------------------------------------------------------------------------
// Functions PROTOTYPES
//-----------------------------------------------------------------------------
void Init_Device(void);
void INIT_TORP(unsigned char id);
void wait(unsigned int ms);
void send_byte(unsigned char byte);
void send_string(unsigned char *string);
void update_data(unsigned char target);
void START(void);
void STOP(void);
void reconfig_servo(void);
unsigned char CRC8(unsigned char *addr, unsigned char len);